@@ -5,6 +5,8 @@ from ...config import LiteLLMConfig
 import os
 from dotenv import load_dotenv
 import logging
+from ai_kit.utils.chunker import Chunkers
+from ai_kit.utils.tokens import TokenCounter
 
 load_dotenv()
 logger = logging.getLogger(__name__)
@@ -88,7 +90,6 @@ class ChatClient(BaseLiteLLMClient):
         self._validate_model(self.model, LiteLLMConfig.SUPPORTED_CHAT_MODELS)
         self.mapped_model = self._get_model_name(self.model)
 
-
     async def chat_completion(
         self, messages: List[Dict[str, str]], stream: bool = False, **kwargs
     ) -> Union[Dict[str, Any], AsyncIterator[Dict[str, Any]]]:
@@ -150,7 +151,6 @@ class ReasoningClient(BaseLiteLLMClient):
                     f"Missing required API key: {required_key} for model {self.model}"
                 )
 
-
     async def reasoning_completion(
         self, messages: List[Dict[str, str]], stream: bool = False, **kwargs
     ) -> Union[Dict[str, Any], AsyncIterator[Dict[str, Any]]]:
@@ -166,9 +166,8 @@ class ReasoningClient(BaseLiteLLMClient):
             )
 
             if stream:
-
                 async def response_generator():
-                    async for chunk in response:
+                    for chunk in response:
                         yield {
                             "choices": [
                                 {
@@ -202,7 +201,6 @@ class ReasoningClient(BaseLiteLLMClient):
         except Exception as e:
             raise Exception(f"Error in reasoning completion: {str(e)}")
 
-
 class EmbeddingResponseData(TypedDict):
     """Data from embedding model."""
 
@@ -251,3 +249,109 @@ class EmbeddingClient(BaseLiteLLMClient):
         except Exception as e:
             logger.error(f"Error creating embedding: {e}")
             raise Exception(f"Error creating embedding: {str(e)}")
+
+
+class RerankResult(TypedDict):
+    """Result from reranking."""
+
+    index: int
+    relevance_score: float
+
+
+class RerankResults(TypedDict):
+    """Results from reranking."""
+
+    results: List[RerankResult]
+    id: str
+    meta: dict
+    billed_units: dict
+
+
+class RerankCompletionResult(TypedDict):
+    """Result from reranking."""
+
+    index: int
+    relevance_score: float
+    document: str  # The document text
+    metadata: Dict[str, str]  # Metadata like title, url etc
+    original_index: int  # Index in original document list
+
+
+class RerankClient(BaseLiteLLMClient):
+    def __init__(self, model: Optional[str] = None):
+        super().__init__(model or LiteLLMConfig.DEFAULT_RERANK_MODEL)
+        self._validate_model(self.model, LiteLLMConfig.SUPPORTED_RERANK_MODELS)
+        self.mapped_model = self._get_model_name(self.model)
+        self.max_tokens_per_document = 4096
+        self.token_counter = TokenCounter()
+
+    async def rerank_completion(
+        self,
+        query: str,
+        documents: List[dict],  # Now expects dicts with "text" and "metadata"
+        top_n: int = 10,
+        autochunk: bool = True,
+    ) -> List[RerankCompletionResult]:
+        from litellm import rerank
+        from ai_kit.utils.chunker import Chunkers
+
+        processed_documents = []
+        for original_index, doc in enumerate(documents):
+            text = doc["text"]
+            metadata = doc["metadata"]
+
+            if self.token_counter.count_tokens(text) > self.max_tokens_per_document:
+                if autochunk:
+                    # Chunk the large document into smaller segments using token-based chunking.
+                    chunks = Chunkers.sliding_window_chunker_by_tokens(
+                        text=text,
+                        chunk_size=self.max_tokens_per_document,
+                        overlap=200,
+                        model=self.token_counter.model,
+                    )
+                    processed_documents.extend(
+                        {
+                            "chunk": chunk,
+                            "metadata": metadata,
+                            "original_index": original_index,
+                        }
+                        for chunk in chunks
+                    )
+                else:
+                    raise Exception("Document is too large and autochunk is disabled.")
+            else:
+                processed_documents.append(
+                    {
+                        "chunk": text,
+                        "metadata": metadata,
+                        "original_index": original_index,
+                    }
+                )
+
+        response: RerankResults = rerank(
+            model=self.mapped_model,
+            query=query,
+            documents=[
+                doc["chunk"] for doc in processed_documents
+            ],  # pass just the text chunks
+            top_n=top_n,
+        )
+
+        if not (results := response.get("results")):
+            raise Exception("No results found")
+        
+        # Combine rerank results with metadata
+        final_results = []
+        for result in results:
+            processed_doc = processed_documents[result["index"]]
+            result["original_index"] = processed_doc["original_index"]
+            result["document"] = processed_doc["chunk"]
+            result["metadata"] = processed_doc["metadata"]
+            final_results.append(RerankCompletionResult(**result))
+
+        sorted_results: List[RerankCompletionResult] = sorted(
+            final_results,
+            key=lambda r: r["relevance_score"],  # sort by relevance score
+            reverse=True,  # in descending order
+        )
+        return sorted_results
