@@ -1,4 +1,4 @@
-from typing import Dict, Any, AsyncIterable, Iterable
+from typing import Dict, Any, AsyncIterable, Iterable, AsyncIterator
 import logging
 import tiktoken
 import re
@@ -28,21 +28,28 @@ def get_headers(chunk: Dict[str, Any]) -> Dict[str, Any]:
     if (h:=chunk.get("_response_headers")): return h
     return {}
 
-async def print_stream(chunks: AsyncIterable[Dict[str, Any]], buffer_size: int = 128):
+async def print_stream(chunks: AsyncIterable[Dict[str, Any]], buffer_size: int = 128) -> str:
     """
-    Read text chunks from an async generator,
-    accumulate them in a buffer, and print to console
-    once the buffer reaches a certain size.
+    Read text chunks from an async generator, print them to console as they arrive,
+    and return the collected text.
+    
+    Args:
+        chunks: Async iterable of chunks to process
+        buffer_size: Size threshold for buffer before printing
+        
+    Returns:
+        The complete text collected from all chunks
     """
     buffer = []  # We'll collect text pieces here
     current_size = 0
+    output = ""
 
     async for chunk in chunks:
         text = get_text(chunk)  # Uses your get_text function
         if text:
             buffer.append(text)
             current_size += len(text)
-
+            output += text
             # If we pass the threshold, print and reset
             if current_size >= buffer_size:
                 print("".join(buffer), end="", flush=True)
@@ -53,6 +60,8 @@ async def print_stream(chunks: AsyncIterable[Dict[str, Any]], buffer_size: int =
     if buffer:
         print("".join(buffer), end="", flush=True)
 
+    return output
+
 def print_stream_synchronous(chunks: Iterable[Dict[str, Any]], buffer_size: int = 128):
     """
     Read text chunks from an sync generator,
